"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const ontology_ts_test_1 = require("ontology-ts-test");
const path = require("path");
const fileSystem_1 = require("../utils/fileSystem");
class Compiler {
    async compile(projectDir, contracts) {
        let contractsPath;
        if (contracts !== undefined) {
            contractsPath = path.join(projectDir, contracts);
            if (!fs.existsSync(contractsPath)) {
                contractsPath = path.join(projectDir, 'contracts', contracts);
            }
        }
        else {
            contractsPath = path.join(projectDir, 'contracts');
        }
        return this.compileInternal(contractsPath);
    }
    async compileInternal(fileOrDir) {
        const stats = fs.statSync(fileOrDir);
        if (stats.isDirectory()) {
            const children = fs.readdirSync(fileOrDir);
            for (const child of children) {
                if (child.startsWith('__') || child.endsWith('.json') || child === 'build') {
                    continue;
                }
                const childPath = path.join(fileOrDir, child);
                await this.compileInternal(childPath);
            }
        }
        else if (stats.isFile()) {
            if (fileOrDir.endsWith('.py') || fileOrDir.endsWith('.cs')) {
                await this.compileContract(path.dirname(fileOrDir), path.basename(fileOrDir));
            }
            else {
                throw Error('Compile Error: file type is wrong');
            }
        }
        else {
            throw new Error('Compile Error: contract path is wrong');
        }
    }
    async compileContract(dir, name, abiSavePath, avmSavePath) {
        const contractPath = path.join(dir, name);
        const code = ontology_ts_test_1.loadContract(contractPath);
        let type;
        let url;
        if (contractPath.endsWith('.py')) {
            type = 'Python';
            url = 'https://smartxcompiler.ont.io/api/beta/python/compile';
        }
        else if (contractPath.endsWith('.cs')) {
            type = 'CSharp';
            url = 'https://smartxcompiler.ont.io/api/v1.0/csharp/compile';
        }
        else {
            throw new Error('Compile Error: contract type is unknown');
        }
        if (abiSavePath === undefined) {
            const splitPath = path.parse(contractPath);
            const savePath = path.join(splitPath.dir, 'build', splitPath.base);
            if (savePath.endsWith('.py')) {
                abiSavePath = savePath.replace('.py', '_abi.json');
            }
            else {
                abiSavePath = savePath.replace('.cs', '_abi.json');
            }
        }
        if (avmSavePath === undefined) {
            const splitPath = path.parse(contractPath);
            const savePath = path.join(splitPath.dir, 'build', splitPath.base);
            if (savePath.endsWith('.py')) {
                avmSavePath = savePath.replace('.py', '.avm');
            }
            else {
                avmSavePath = savePath.replace('.cs', '.avm');
            }
        }
        // disable SSL verify because of misconfigured compiler server
        process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';
        const result = await ontology_ts_test_1.compile({ code, type, url });
        process.env.NODE_TLS_REJECT_UNAUTHORIZED = '1';
        fileSystem_1.ensureDirExist(path.parse(avmSavePath).dir);
        fileSystem_1.ensureDirExist(path.parse(abiSavePath).dir);
        fs.writeFileSync(avmSavePath, result.avm.toString('hex'));
        fs.writeFileSync(abiSavePath, result.abi);
    }
}
exports.Compiler = Compiler;
//# sourceMappingURL=compiler.js.map