"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ontology_ts_test_1 = require("ontology-ts-test");
const path = require("path");
const configLoader_1 = require("../config/configLoader");
const punicaException_1 = require("../exception/punicaException");
const async_1 = require("../utils/async");
const cliUtils_1 = require("../utils/cliUtils");
const fileSystem_1 = require("../utils/fileSystem");
const params_1 = require("../utils/params");
// tslint:disable:no-console
class Invoker {
    list(projectDir, configKey) {
        const invokeConfig = configLoader_1.loadInvoke(projectDir, configKey);
        return invokeConfig.functions.map((func) => func.operation);
    }
    async invoke(projectDir, execFuncStr, networkKey, walletFileName, configKey, debug) {
        const rpcAddress = configLoader_1.loadNetwork(projectDir, networkKey);
        const invokeConfig = configLoader_1.loadInvoke(projectDir, configKey);
        const [wallet] = configLoader_1.loadWallet(projectDir, walletFileName);
        const client = ontology_ts_test_1.initClient({ rpcAddress });
        const abiFileName = invokeConfig.abi;
        if (abiFileName === undefined) {
            throw punicaException_1.configFileError();
        }
        console.log(`Running invocation: ${abiFileName}`);
        const defaultPayer = invokeConfig.defaultPayer;
        let defaultPassword;
        if (defaultPayer !== undefined) {
            console.log('Unlock default payer account...');
            const configPassword = configLoader_1.loadPassword(projectDir, configKey, defaultPayer);
            if (configPassword !== undefined) {
                defaultPassword = configPassword;
            }
            else {
                defaultPassword = await async_1.questionAsync('Please input payer account password: ');
            }
        }
        const abiDirPath = path.join(projectDir, 'contracts', 'build');
        const abi = fileSystem_1.readAbi(abiDirPath, abiFileName);
        if (abi.hash === undefined) {
            throw punicaException_1.abiFileError();
        }
        const contractAddress = abi.hash;
        const deployed = await ontology_ts_test_1.isDeployed({ client, scriptHash: contractAddress });
        if (!deployed) {
            console.log(`Contract ${contractAddress} hash't been deployed to current network ${networkKey}`);
            return;
        }
        const invokeFunctions = new Map();
        for (const func of invokeConfig.functions) {
            invokeFunctions.set(func.operation, func);
        }
        const abiFunctions = new Map();
        for (const func of abi.functions) {
            abiFunctions.set(func.name, func);
        }
        let allExecFuncs;
        if (execFuncStr !== undefined) {
            allExecFuncs = execFuncStr.split(',');
        }
        else {
            allExecFuncs = Array.from(invokeFunctions.keys());
        }
        const sleepTime = invokeConfig.sleepTime !== undefined ? invokeConfig.sleepTime * 1000 : 6000;
        for (const functionName of allExecFuncs) {
            cliUtils_1.wrapDebug(debug, async () => {
                const invokeInfo = invokeFunctions.get(functionName);
                if (invokeInfo === undefined) {
                    throw punicaException_1.otherError(`There is no function with name ${functionName} in the config.`);
                }
                const abiInfo = abiFunctions.get(functionName);
                if (abiInfo === undefined) {
                    throw punicaException_1.otherError(`There is no function with name ${functionName} in the ABI file.`);
                }
                const invokeParams = invokeInfo.args !== undefined ? invokeInfo.args : [];
                console.log(`Invoking ${functionName}...`);
                if (abiInfo.parameters.length !== Object.keys(invokeParams).length) {
                    throw punicaException_1.otherError(`Invoke failed, params mismatch between config and ABI file.`);
                }
                const parameters = params_1.convertParams(invokeParams, abiInfo);
                if (invokeInfo.preExec) {
                    const response = await ontology_ts_test_1.invoke({
                        client,
                        contract: contractAddress,
                        method: abiInfo.name,
                        parameters,
                        gasLimit: String(invokeConfig.gasLimit),
                        gasPrice: String(invokeConfig.gasPrice),
                        preExec: true,
                        debug
                    });
                    const result = response.result;
                    if (result !== undefined) {
                        const inner = result.Result;
                        if (inner !== undefined) {
                            console.log('Invocation was successful. Result:', inner);
                            return;
                        }
                    }
                    console.warn('Invocation failed.');
                }
                else {
                    const payer = invokeInfo.payer !== undefined ? invokeInfo.payer : defaultPayer;
                    if (payer === undefined) {
                        throw punicaException_1.otherError('Missing payer.');
                    }
                    const account = configLoader_1.loadAccount(wallet, payer);
                    let password;
                    if (payer === defaultPayer && defaultPassword !== undefined) {
                        password = defaultPassword;
                    }
                    else {
                        const configMethodPassword = configLoader_1.loadPassword(projectDir, configKey, payer);
                        if (configMethodPassword !== undefined) {
                            password = configMethodPassword;
                        }
                        else {
                            password = await async_1.questionAsync('Please input payer account password: ');
                        }
                    }
                    // add additional signatures
                    let signers;
                    const signature = invokeInfo.signature;
                    if (signature !== undefined && signature.m !== undefined && signature.signers !== undefined) {
                        signers = [];
                        for (const signer of signature.signers) {
                            const signerAccount = configLoader_1.loadAccount(wallet, signer);
                            let signerPassword = configLoader_1.loadPassword(projectDir, configKey, signer);
                            if (signerPassword === undefined) {
                                signerPassword = await async_1.questionAsync(`Please input password for account ${signer} : `);
                            }
                            signers.push({ account: signerAccount, password: signerPassword });
                        }
                    }
                    const response = await ontology_ts_test_1.invoke({
                        client,
                        account,
                        password,
                        signers,
                        contract: contractAddress,
                        method: abiInfo.name,
                        parameters,
                        wait: false,
                        gasLimit: String(invokeConfig.gasLimit),
                        gasPrice: String(invokeConfig.gasPrice)
                    });
                    await sleep(sleepTime);
                    const result = response.result;
                    if (result !== '') {
                        console.log(`Invocation was successful. Transaction: ${result}.`);
                    }
                    else {
                        console.warn(`Invocation failed. Error: ${result}.`);
                    }
                }
            });
        }
    }
}
exports.Invoker = Invoker;
function sleep(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}
//# sourceMappingURL=invoker.js.map