"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const configLoader_1 = require("../config/configLoader");
const punicaException_1 = require("../exception/punicaException");
const cliUtils_1 = require("../utils/cliUtils");
const fileSystem_1 = require("../utils/fileSystem");
const params_1 = require("../utils/params");
// tslint:disable-next-line:no-var-requires
const jest = require('jest-cli');
// tslint:disable:no-console
// tslint:disable:quotemark
class Tester {
    async exec(punicaLibDir, projectDir, fileName) {
        const punicaRootDir = punicaLibDir.substr(0, punicaLibDir.length - '/lib'.length);
        const root = path.join(projectDir, 'test');
        const file = path.join(root, fileName);
        const options = {
            roots: [root],
            config: punicaRootDir,
            testRegex: file
        };
        // console.log('projectDir:', projectDir);
        // console.log('roots:', options.roots);
        // console.log('cwd:', process.cwd());
        // console.log('dirname:', punicaRootDir);
        process.chdir(projectDir);
        await jest.runCLI(options, [root]);
    }
    template(projectDir, configKey, abiPath, walletFileName, debug) {
        const templatePath = `${__dirname}/../../template/sc.test.ts`;
        const invokeConfig = configLoader_1.loadInvoke(projectDir, configKey);
        const [, walletPath] = configLoader_1.loadWallet(projectDir, walletFileName);
        const abiFullPath = path.join(projectDir, abiPath);
        const abiName = path.basename(abiFullPath);
        const abiDir = path.dirname(abiFullPath);
        const className = abiName.replace('_abi.json', '');
        const testDirPath = path.join(projectDir, 'test');
        const testFilePath = path.join(testDirPath, `${className}.test.ts`);
        const walletRelativePath = path.relative(projectDir, walletPath);
        const abi = fileSystem_1.readAbi(abiDir, abiName);
        const data = fs.readFileSync(templatePath, 'utf8');
        const defaultPayer = invokeConfig.defaultPayer;
        const tests = this.buildMethodTests(projectDir, configKey, defaultPayer, invokeConfig, abi, debug);
        const replaced = this.replacePlaceholders(data, abi.hash, walletRelativePath, tests);
        fs.writeFileSync(testFilePath, replaced, 'utf8');
    }
    replacePlaceholders(data, contract, walletPath, tests) {
        const testsStr = tests.join('\n\n');
        data = data.replace('${contract}', contract);
        data = data.replace('${walletPath}', walletPath);
        data = data.replace('${tests}', testsStr);
        return data;
    }
    buildMethodTests(projectDir, configKey, defaultPayer, invokeConfig, abi, debug) {
        const invokeFunctions = new Map();
        for (const func of invokeConfig.functions) {
            invokeFunctions.set(func.operation, func);
        }
        const abiFunctions = new Map();
        for (const func of abi.functions) {
            abiFunctions.set(func.name, func);
        }
        const allExecFuncs = Array.from(invokeFunctions.keys());
        const tests = [];
        for (const functionName of allExecFuncs) {
            cliUtils_1.wrapDebug(debug, async () => {
                const invokeInfo = invokeFunctions.get(functionName);
                if (invokeInfo === undefined) {
                    throw punicaException_1.otherError(`There is no function with name ${functionName} in the config.`);
                }
                const abiInfo = abiFunctions.get(functionName);
                if (abiInfo === undefined) {
                    throw punicaException_1.otherError(`There is no function with name ${functionName} in the ABI file.`);
                }
                const invokeParams = invokeInfo.args !== undefined ? invokeInfo.args : [];
                console.log(`Generating test for ${functionName}...`);
                if (abiInfo.parameters.length !== Object.keys(invokeParams).length) {
                    throw punicaException_1.otherError(`Invoke failed, params mismatch between config and ABI file.`);
                }
                const parameters = params_1.convertParamsStr(invokeParams, abiInfo);
                const parametersStr = parameters.join(', ');
                if (invokeInfo.preExec) {
                    const methodCall = `test('test ${functionName}', async () => {\n` +
                        `   const response = await invoke({\n` +
                        `     client,\n` +
                        `     contract,\n` +
                        `     preExec: true,\n` +
                        `     method: '${abiInfo.name}',\n` +
                        `     parameters: [${parametersStr}]\n` +
                        `   });\n` +
                        ` console.log(response);\n` +
                        ` });\n`;
                    tests.push(methodCall);
                }
                else {
                    const payer = invokeInfo.payer !== undefined ? invokeInfo.payer : defaultPayer;
                    if (payer === undefined) {
                        throw punicaException_1.otherError('Missing payer.');
                    }
                    let password = configLoader_1.loadPassword(projectDir, configKey, payer);
                    if (password === undefined) {
                        password = '';
                    }
                    const methodCall = `  test('test ${functionName}', async () => {\n` +
                        `    const response = await invoke({\n` +
                        `      client,\n` +
                        `      account: wallet.getAccount('${payer}'),\n` +
                        `      password: '${password}',\n` +
                        `      gasLimit,\n` +
                        `      gasPrice,\n` +
                        `      contract,\n` +
                        `      method: '${abiInfo.name}',\n` +
                        `      parameters: [${parametersStr}]\n` +
                        `    });\n` +
                        `    console.log(response);\n` +
                        `  });\n`;
                    tests.push(methodCall);
                }
            });
        }
        return tests;
    }
}
exports.Tester = Tester;
//# sourceMappingURL=tester.js.map