"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const ontology_ts_crypto_1 = require("ontology-ts-crypto");
const path = require("path");
const uuid = require("uuid");
const punicaException_1 = require("../exception/punicaException");
const fileSystem_1 = require("../utils/fileSystem");
const walletCli_1 = require("./walletCli");
class WalletManager {
    init(projectDir, walletFileName, create) {
        if (walletFileName === undefined) {
            walletFileName = path.join('wallet', 'wallet.json');
        }
        const walletPath = path.join(projectDir, walletFileName);
        const walletPathDir = path.dirname(walletPath);
        if (create && !fs.existsSync(walletPathDir)) {
            fileSystem_1.ensureDirExist(walletPathDir);
        }
        if (fs.existsSync(walletPath)) {
            this.loadWallet(walletPath);
        }
        else {
            this.createWallet(walletPath);
        }
    }
    async addAccount() {
        const password = await walletCli_1.inputNewPassword();
        const account = ontology_ts_crypto_1.Account.create(uuid(), ontology_ts_crypto_1.PrivateKey.random(), password, this.wallet.scrypt);
        this.wallet.addAccount(account);
        this.saveWallet();
    }
    async deleteAccount(address) {
        const password = await walletCli_1.inputExistingPassword();
        const account = this.wallet.getAccount(address);
        if (account === undefined) {
            throw punicaException_1.otherError(`Account with address ${address} does not exist.`);
        }
        // try to decrypt
        await account.decryptKey(password);
        this.wallet.delAccount(address);
        this.saveWallet();
    }
    async importAccount(sk) {
        const password = await walletCli_1.inputNewPassword();
        const account = ontology_ts_crypto_1.Account.import(uuid(), new ontology_ts_crypto_1.PrivateKey(sk), password, undefined, this.wallet.scrypt);
        this.wallet.addAccount(account);
        this.saveWallet();
    }
    listAccounts() {
        if (this.wallet.accounts !== undefined) {
            return this.wallet.accounts.map((a) => a.address.toBase58());
        }
        else {
            return [];
        }
    }
    async addIdentity() {
        const password = await walletCli_1.inputNewPassword();
        const identity = ontology_ts_crypto_1.Identity.create(uuid(), ontology_ts_crypto_1.PrivateKey.random(), password, this.wallet.scrypt);
        this.wallet.addIdentity(identity);
        this.saveWallet();
    }
    async deleteIdentity(ontid) {
        const password = await walletCli_1.inputExistingPassword();
        const identity = this.wallet.getIdentity(ontid);
        if (identity === undefined) {
            throw punicaException_1.otherError(`Identity with ontid ${ontid} does not exist.`);
        }
        // try to decrypt
        await identity.decryptKey('1', password);
        this.wallet.delIdentity(ontid);
        this.saveWallet();
    }
    listIdentities() {
        if (this.wallet.identities !== undefined) {
            return this.wallet.identities.map((a) => a.ontid);
        }
        else {
            return [];
        }
    }
    loadWallet(walletPath) {
        try {
            const f = fs.readFileSync(walletPath, 'utf8');
            this.wallet = ontology_ts_crypto_1.Wallet.deserializeJson(f);
            this.walletPath = walletPath;
        }
        catch (e) {
            throw punicaException_1.walletFileError();
        }
    }
    saveWallet() {
        try {
            const data = this.wallet.serializeJson(true);
            fs.writeFileSync(this.walletPath, data, 'utf8');
        }
        catch (e) {
            throw punicaException_1.walletFileError();
        }
    }
    createWallet(walletPath) {
        try {
            this.wallet = ontology_ts_crypto_1.Wallet.create();
            fs.writeFileSync(walletPath, 'utf8');
            this.walletPath = walletPath;
        }
        catch (e) {
            throw punicaException_1.walletFileError();
        }
    }
}
exports.WalletManager = WalletManager;
//# sourceMappingURL=walletManager.js.map